!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.2
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
program test_grib

! PROGRAM TO UNIT TEST THE GRIB WRITING LIBRARY IN LIS
!
! CHARLES ALONGE 12/5/2007 - INITIAL SPECIFICATION
!
! PROGRAM READS IN A BIG ENDIAN SEQUENTIAL ACCESS BINARY
! FILE CONTAINING NLDAS TEMPERATURE DATA AND THEN PACKS
! THAT DATA INTO A GRIB FILE.  TESTS THE BASIC FUNCTIONALITY
! OF THE GRIB LIBRARY BY WRITING A SINGLE MESSAGE THAT USES
! A LOGICAL BITMAP TO PACK ONLY THE VALID DATA POINTS

implicit none

! Date information for grib file
integer     :: imin
integer     :: ihour
integer     :: iday
integer     :: imon 
integer     :: iyear

! Grid dimensions for input data
integer     :: imax
integer     :: jmax

! Grib specific data arrays
integer               :: sect1(36) !PDS Sections
integer               :: sect2(25) !GDS Section
integer               :: s1opt(1)  !Optional PDS data
integer, allocatable  :: sect3(:)  !Bitmap section

! Arrays to hold the input data
integer, allocatable  :: land(:)
real, allocatable     :: nldastmp(:,:)
real, allocatable     :: nldasthin(:)

! Grib processing data
integer     :: info(5)      ! Header information
integer     :: temp(8)      ! For packing into Octets
integer     :: size_sect3   ! Size of Section 3 (bitmap)
integer     :: numoct_sect3 ! Number of Octets in Section 3 total 
integer     :: numpts       ! Number of data points to pack
integer     :: grid_proj    ! Grid projection identifier
integer     :: octets       ! Hold number of octects
integer     :: sect3_index  ! For packing bitmap 
integer     :: index,index_i ! For packing bitmap
real        :: calc_octs   ! Real for calculating # of octets

! General loop counters
integer     :: i,j,k,count,lcount

!Set dimensions of input data
imax = 464
jmax = 224

!Set Date Information
iyear = 1985
imon = 4
iday = 15
ihour = 18
imin = 0

!Allocate NLDAS specific arrays
!for input from binary file
allocate(nldastmp(imax,jmax))
allocate(land(imax*jmax))

!Open input data (big endian sequential)
open(unit=10,file="gribwritertestinput.bin",form="unformatted")
read(10) nldastmp
close(10)

!Open process input data and build a landmask
!based on the number of undefined points
numpts = 0
count=0
do j=1,jmax
   do i=1,imax
     land(count+i) = 0
     if(nldastmp(i,j).gt.9000) nldastmp(i,j) = -9999.00 
     if(nldastmp(i,j).ne.-9999.00) then
        land(count+i) = 1
        numpts = numpts + 1
     endif     
   enddo
   count = count + imax
enddo

!Allocate an array to hold only the valid data
allocate(nldasthin(numpts))

!Populate the valid data array (i.e. land data only)
!bitmap in grib file will mask out the rest.  This is
!useful as it will reduce storage space needed for data
count=0
lcount=0
do j=1,jmax
   do i=1,imax
      if( land(count+i) .eq. 1 ) then
         lcount = lcount + 1
         nldasthin(lcount) = nldastmp(i,j)
      endif     
   enddo
   count = count + imax
enddo

!Initialize header information to zero
info  = 0

! info(1) set in GRIBDAT routine
info(2) = 0
info(3) = 32

!Setup Section 1 of grib file - Parameter Data
!Contains information about the field and time
!Elements 25-36 reserved - set to zero
sect1 = 0
sect1(1)  = 130  ! Table Number
sect1(2)  = 7    ! Center Number
sect1(3)  = 1    ! Process ID
sect1(4)  = 255  ! grid number - 255 undefined
sect1(5)  = 192  ! 192 - bitmapped data, 128 - otherwise
sect1(6)  = 11   ! Parameter ID 11=Temperature
sect1(7)  = 1    ! level/layer type
sect1(8)  = 0    ! layer top
sect1(9)  = 0    ! layer bottom
sect1(10) = 1    ! level/layer flag

!Determine year
sect1(11) = mod(iyear,100)
if (sect1(11) .eq. 0) sect1(11) = 100

!Set other date information
!See grib tables for more information
sect1(12) = imon
sect1(13) = iday
sect1(14) = ihour
sect1(15) = imin

sect1(16) = 0    ! forecast time unit
sect1(17) = 0    ! time period 1
sect1(18) = 0    ! time period 2
sect1(19) = 1    ! time range indicator
sect1(20) = 0    ! # included in avg
sect1(21) = 0    ! # missing in avg

!Determine century 
if(sect1(11) == 100) then
  sect1(22) = iyear / 100 
else
  sect1(22) = (iyear / 100) + 1
endif

! Miscellaneous infor
sect1(23) = 138  !subcenter id
sect1(24) = 2    !scaling factor, decimal precision

!Setup Section 2 of grib file - Grid Metadata
!Contains information about the grid projection
!Note: elements 11 and up are dependent upon
!the map projection of the data
sect2 = 0
sect2(1) = 0       ! Vertical Coord Params
sect2(2) = 255     ! Grid Number  
sect2(3) = 0       ! Map Projection - 0 is Lat/Lon 
sect2(4) = imax    ! E-W Size of Grid 
sect2(5) = jmax    ! N-S Size of Grid
sect2(6) = 0       ! Hemisphere flag 0-NH
sect2(7) = 25063   ! Lat of (1,1) 
sect2(8) = 1       ! Hemsphere flag 1-WH 
sect2(9) = 124938  ! Lon of (1,1) 
sect2(10)= 128     ! Resolution/Component Flags

!Rest depends on projection, Lat/Lon in this case
sect2(11)= 0       ! Hemis flag 0-NH
sect2(12)= 52938   ! Lat of upper right point
sect2(13)= 1       ! Hemis Flag 1-WH
sect2(14)= 67063   ! Lon of upper right point
sect2(15)= 125     ! Grid increment lon
sect2(16)= 125     ! Grid increment lat
sect2(17)= 64      ! Scanning mode flags
sect2(18)= 0       ! Reserved, set to zero

!Setup Section 3 of grib file
!Contains information about the bitmap used
!in outputting the data and also contains 
!information about the lenght of the header
!message
calc_octs = (imax*jmax) / 8.0
octets   = ceiling(calc_octs)
numoct_sect3  = octets + 6
size_sect3    = octets + 2

!Fill in rest of header information
info(4) = numoct_sect3
info(5) = numpts

allocate(sect3(size_sect3))

!Find out how many octets will encapsulate
!the total amount of input data in order
!to create an output bitmap 
sect3(1) = mod(imax*jmax,8)
sect3(2) = 0
 
!----------------------------------------------------------------------
!loop through each point in the land mask array (actually loop
!through 8 points at a time).  read in the zeros and ones as a
!binary number and convert to the decimal equivalent.  a one
!indicates land exists at that point...a zero indicates water.
!----------------------------------------------------------------------
do i=1,imax*jmax,8

   do index = 1,8     
      index_i = i+index-1

      if(index_i .gt. imax*jmax) then
         temp(index) = 0
      else
         temp(index) = land(index_i)
      endif
 
   enddo          

!----------------------------------------------------------------------
!for each octet processed, determine the appropriate index
!for storage in the section 3 bitmap section.  the first
!octet is stored in array position 3, the second in array
!position 4, and so on...
!----------------------------------------------------------------------

   sect3_index = i/8 + 3

   !write(logunit,*) "sect3 index",sect3_index,i,j,size_3,imax,jmax
    sect3(sect3_index) = temp(1)*128 + temp(2)*64 + temp(3)*32 + &
                     temp(4)*16 + temp(5)*8 + temp(6)*4 + &
                     temp(7)*2 + temp(8)

enddo

!Open up grib file for writing
open(12,file="test.grb",form='unformatted')

!Call preprocessing routines and pack grib message
call gribdat(info, sect1, s1opt, sect2, sect3, nldasthin, &
              0, 0, numpts, 1, 1, size_sect3)

!Close grib file for writing
close(12)

!Deallocate allocatables
deallocate(sect3)
deallocate(nldastmp)
deallocate(land)

!We're done 
stop

end

SUBROUTINE GRIBDAT (INFO, SECT1, S1OPT, SECT2, SECT3, DUM3D, &
                    TOPLEV, BOTLEV, IDIM, JDIM, KDIM, SIZE_3)
!
! !DESCRIPTION: 
!  This routine calls the low-level grib routines, contained 
!  in the grib library, to pack the grib record corresponding to the
!  variable contained in dum3d.  Then it writes the grib record.
!
!   The arguments are:
!   \begin{description}
!   \item [info]
!     Sizes of grib sections
!   \item [sect1]
!     Parameters for Section 1 (PDS) of the GRIB record
!   \item [s1opt]
!     Optional parameters for Section 1 (PDS) of the GRIB record
!   \item [sect2]
!     Parameters for Section 2 (GDS) of the GRIB record
!   \item [sect3]
!     Parameters for Section 3 (BMS) of the GRIB record
!   \item [dum3d]
!     Array of data to be processed
!   \item [toplev]
!     Array of depths for the top of each layer
!   \item [botlev]
!     Array of depths for the bottom of each layer
!   \item [idim]
!     Number of points in the i-direction
!   \item [jdim]
!     Number of points in the j-direction
!   \item [kdim]
!     Number of layers
!   \item [size\_3]
!     Number of elements in the sect3 array.
!   \end{description}
!EOP

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!--   
!--   NAME: GRIB DATA
!--   ====
!--
!--   PURPOSE
!--   =======
!--   CALL GRIB UTILITIES AND WRITE OUT THE GRIB FILE
!--
!--   METHOD:
!--   ======
!--   1. DETERMINE THE SIZE OF THE GRIB MESSAGE
!--   2. CALL GRIB ROUTINES TO BUILD GRIB MESSAGE
!--   3. WRITE GRIB MESSAGE TO FILE
!--   
!--   REFERENCE
!--   =========
!--   NCEP GRIB MANUAL OFFICE NOTE 388
!--
!--   PROCESS NARRATIVE:
!--   =================
!--   NONE
!--
!--   INTERFACES:
!--   ==========
!--     INPUTS  : INFO, SECT1, SECT2, SECT3, S1OPT, DUM3D, TOPLEV,
!--               BOTLEV, IDIM, JDIM, KDIM
!--
!--     OUTPUTS : NONE
!--
!--   CALLED FROM:
!--   ===========
!--   AVST, EVAP, EVAP_SFC, RAIN, SAVSNO, SOILMS, SOILMS_SFC, SUMRAD,
!--   TEMPS 
!--
!--   FILES ACCESSED:
!--   ==============
!--   NONE   
!--
!--   VARIABLES
!--   =========...................DESCRIPTION..........................
!--   BITNUM   STARTING BIT OF WORDNO TO LOAD GRIB INFO
!--   BOTLEV   HOLDS ARRAY OF SECT1(9) VALUES
!--   BUFFSZ   DIMENSION OF GRBOUT ARRAY
!--   CSTAT    I/O STATUS
!--   DUM3D    INPUT ARRAY OF DATA TO BE GRIBBED
!--   GRBOUT   ARRAY THAT HOLDS GRIBBED FIELD
!--   I        LOOP COUNTER
!--   IDIM     I DIMENSION OF GRIBBED FIELD
!--   IMIN     MINIMUM VALUE OF FIELD TO BE GRIBBED
!--   INFO     SIZES OF GRIB SECTIONS
!--              1 - SIZE IN BITS/32 OF ENTIRE GRIB MESSAGE
!--              2 - SIZE OF SECTION 1 OPTIONAL INFORMATION IN OCTETS
!--              3 - SIZE OF SECTION 2 IN OCTETS
!--              4 - SIZE OF SECTION 3 IN OCTETS
!--              5 - NUMBER OF DATA POINTS BEING GRIBBED
!--   INGRID   FIELD TO BE GRIBBED AFTER SCALING AND
!--            CONVERSION TO INTEGER
!--   ISIZE    SIZE OF EACH GRIB SECTION IN BITS
!--   ISTAT    I/O STATUS
!--   ISTAT1   I/O STATUS
!--   ITOTBT   NUMBER OF BITS FOR ENTIRE GRIB RECORD
!--   J        LOOP COUNTER
!--   JDIM     J DIMENSION OF GRIBBED FIELD
!--   K        LOOP INDEX
!--   KDIM     K DIMENSION OF GRIBBED FIELD
!--   NBTGRD   MINIMUM NUMBER OF BITS REQUIRED FOR FIELD BEING GRIBBED
!--   OUTBIT   ENDING BIT OF OUTWRD
!--   OUTWRD   ENDING WORD OF GRBOUT ARRAY
!--   SCALE    SCALING FACTOR FOR GRIBBED FIELD
!--   SECT1    SECTION 1 INFORMATION (SEE GRIB MANUAL FOR DETAILS)
!--            GRIB UTILITY SETS OCTS 1,3
!--              1 - PARAMETER TABLE VERSION NUMBER     - OCT 4
!--              2 - AFWA ID NUMBER                     - OCT 5
!--              3 - MODEL ID NUMBER                    - OCT 6
!--              4 - GRID ID NUMBER                     - OCT 7
!--              5 - GDS AND SMP FLAG                   - OCT 8
!--              6 - PARAMETER AND UNIT TYPE            - OCT 9
!--              7 - LEVEL/LAYER TYPE                   - OCT 10
!--              8 - LAYER TOP WHEN SECT1(10) IS 0   
!--                  STORED IN VARIABLE TOPLEV          - OCT 11
!--              9 - LAYER BOTTOM WHEN SECT1(10) IS 0 
!--                  STORED IN VARIABLE BOTLEV          - OCT 11,12
!--              9 - LEVEL WHEN SECT1(10) IS 1      
!--                  STORED IN VARIABLE BOTLEV          - OCT 11,12
!--             10 - LEVEL (1) / LAYER (0) FLAG FOR GRIB
!--                  UTILITY.  FOR LAYERED FIELDS
!--                  (E.G. 1000-500 MB THICKNESS), GRIB PLACES
!--                  LAYER TOP (SECT1(8)) IN OCT 11 AND LAYER
!--                  BOTTOM (SECT1(9)) IN OCT 12.  FOR
!--                  FIELDS AT A SINGLE LEVEL, LEVEL IS
!--                  PLACED IN SECT1(9) AND GRIB UTILITY PLACES
!--                  IT IN 2 OCTETS (11 AND 12), AND
!--                  SECT1(8) IS NOT USED.  NOTE: INFO
!--                  IN SECT1(10) IS NOT GRIBBED.  SEE
!--                  TABLE 3 IN GRIB REFERENCE AND GRIB
!--                  UTILITY FOR MORE DETAILS.
!--             11 - CYCLE YEAR                         - OCT 13
!--             12 - CYCLE MONTH                        - OCT 14
!--             13 - CYCLE DAY                          - OCT 15
!--             14 - CYCLE HOUR                         - OCT 16
!--             15 - CYCLE MINUTE                       - OCT 17
!--             16 - FCST TIME UNIT                     - OCT 18
!--             17 - TIME PERIOD 1                      - OCT 19
!--             18 - TIME PERIOD 2                      - OCT 20
!--             19 - TIME RANGE INDICATOR               - OCT 21
!--             20 - NUMBER INCLUDED IN AVG WHEN OCT 21
!--                  INDICATES AVG OR ACCUM             - OCT 22,23
!--             21 - NUMBER MISSING FROM AVGS OR ACCUM  - OCT 24
!--             22 - CENTURY                            - OCT 25
!--             23 - SUBCENTER                          - OCT 26
!--             24 - SCALING FACTOR                     - OCT 27,28
!--          25-36 - RESERVED (SET TO ZERO)             - OCT 29,40
!--   SECT2    SECTION 2 INFORMATION
!--            SEE GRIB MANUAL FOR MORE DETAILS
!--            GRIB UTILITY SETS OCTS 1,3
!--              1 - VERTICAL COORDINATE PARAMS         - OCT 4
!--              2 - NOT USED                           - OCT 5
!--              3 - MAP PROJECTION GRIB CONVENTION     - OCT 6
!--                   1 - MERCATOR
!--                   3 - LAMBERT CONF
!--                   5 - POLAR STEREOGRAPHIC
!--              4 - GRID DIM I - DIR                   - OCT 7,8
!--              5 - GRID DIM J - DIR                   - OCT 9,10
!--              6 - HEMISPHERE FLAG  (1-SH 0-NH)       - OCT 11
!--              7 - LATITUDE OF POINT (1,1)            - OCT 12,13
!--              8 - HEMISPHERE FLAG  (1-WH 0-EH)       - OCT 14
!--              9 - LONGITUDE OF POINT (1,1)           - OCT 15,16
!--             10 - RESOLUTION AND COMPONENT FLAGS     - OCT 17
!--            
!--             REMAINDER OF SECTION 2 DEPENDS ON MAP PROJECTION -
!--           
!--             LAMBERT CONFOMAL PROJECTION.
!--             11 - HEMISPHERE FLAG FOR ORIENT ANGLE   - OCT 18
!--             12 - ORIENTATION ANGLE (MILLIDEGREES)   - OCT 19,20
!--             13 - HORIZ RESOLUTION (METERS) X-DIR    - OCT 21,23
!--             14 - HORIZ RESOLUTION (METERS) Y-DIR    - OCT 24,26
!--             15 - CENTER PROJECTION FLAG             - OCT 27
!--             16 - SCANNING MODE FLAGS                - OCT 28
!--             17 - HEMIS FLAG FOR SECT2(18)           - OCT 29
!--             18 - TRUE LATIT CLOSEST TO POLE (M DEG) - OCT 30,31
!--             19 - HEMIS FLAG FOR SECT2(20)           - OCT 32
!--             20 - TRUE LATIT 2ND FROM POLE (M DEG)   - OCT 33,34
!--             21 - HEMISPHERE FLAG FOR SECT2(22)      - OCT 35
!--             22 - LATITUDE OF S POLE  (M DEG)        - OCT 36,37
!--             23 - HEMISPHERE FLAG FOR SECT2(24)      - OCT 38
!--             24 - LONGITUDE OF S POLE (M DEG)        - OCT 39,40
!--             25 - RESERVED                           - OCT 41,42
!--
!--             POLAR STEREOGRAPHIC PROJECTION.
!--             11 - HEMISPHERE FLAG FOR ORIENT ANGLE   - OCT 18
!--             12 - ORIENTATION ANGLE (MILLIDEGREES)   - OCT 19,20
!--             13 - HORIZ RESOLUTION (METERS) X-DIR    - OCT 21,23
!--             14 - HORIZ RESOLUTION (METERS) Y-DIR    - OCT 24,26
!--             15 - PROJECTION CENTER FLAG             - OCT 27
!--             16 - SCANNING MODE FLAGS                - OCT 28
!--             17 - RESERVED                           - OCT 29,32
!--
!--             MERCATOR PROJECTION.
!--             11 - HEMIS FLAG FOR SECT2(12)           - OCT 18
!--             12 - LAT OF LAST GRID POINT (M DEG)     - OCT 19,20
!--             13 - HEMIS FLAG FOR SECT2(13)           - OCT 21
!--             14 - LONG OF LAST GRID POINT (M DEG)    - OCT 22,23
!--             15 - TRUE LATITUDE (ALWAYS EQUATOR
!--                                 FOR MM5 MERC)       - OCT 24,26
!--             16 - RESERVED (SET TO ZERO)             - OCT 27
!--             17 - SCANNING MODE FLAGS                - OCT 28
!--             18 - HORIZ GRID RES X-DIR (METERS)      - OCT 29,31
!--             19 - HORIZ GRID RES Y-DIR (METERS)      - OCT 32,34
!--             20 - RESERVED (SET TO ZERO)             - OCT 35,42
!--   SECT3     SECTION 1 INFORMATION (SEE GRIB MANUAL FOR DETAILS)
!--             GRIB UTILITY SECT OCTS1,3
!--              1 - number of unused bits at end of sect3 - oct 4
!--              2 - set to zero if bitmap follows, set
!--                  to numeric if center has predefined
!--                  bitmap                             - oct 5, 6
!--              3 -> nnn - actual bitmap section if sect3(2) 
!--                  equals zero                        - oct 7 ->nnn
!--   SIZE3    DIMENSION NEEDED FOR SECTION 3 ARRAY.  THIS IS
!--            BITMAP_OCT + 2 (THE ADDITIONAL 2 ELEMENTS EQUATE
!--            TO OCTETS 4 AND OCTETS 5-6)
!--   S1OPT    OPTIONAL SECTION 1 INFORMATION (NOT USED)
!--   TOPLEV   HOLDS ARRAY OF SECT1(8) VALUES
!--   WORDNO   STARTING WORD OF ARRAY GRBOUT TO LOAD GRIB INFO
!--
!--   REMARKS:
!--   =======
!--   NONE
!--
!--   UPDATES
!--   =======
!--   NOV ?? 97  INITIAL VERSION (FROM MMPOST).....................DNXM
!--   FEB 03 99  ADAPTED FOR USE IN MMLITE.........................DNXM
!--   MAY 08 00  MODIFIED FOR USE IN AGRMET SYSTEM....CAPT HIDALGO/DNXM
!--   AUG 07 00  CHANGED VARIABLE SECT2 TO BE AN ADJUSTABLE ARRAY
!--              TO HANDLE THE DIFFERENT REQUIREMENTS FOR LAT/LON
!--              AND POLAR STEREOGRAPHIC GRIDS............MR GAYNO/DNXM
!--   DEC 21 06  ADDED FEATURES FOR LIS..................C. ALONGE/SAIC
!--
!----------------------------------------------------------------------
!----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER                      :: BITNUM
      INTEGER                      :: BUFFSZ
      INTEGER,   ALLOCATABLE       :: GRBOUT  ( : )
      INTEGER                      :: I
      INTEGER,   INTENT(IN)        :: IDIM
      INTEGER,   INTENT(IN)        :: JDIM
      INTEGER,   PARAMETER         :: IGRIB = 12
      INTEGER                      :: IMIN
      INTEGER,   INTENT(INOUT)     :: INFO    ( 5 )
      INTEGER                      :: INGRID(IDIM*JDIM)
      INTEGER                      :: ISIZE   ( 0:5 )
      INTEGER                      :: ISTAT
      INTEGER                      :: ISTAT1
      INTEGER                      :: ITOTBT
      INTEGER                      :: J
      INTEGER                      :: K
      INTEGER,   INTENT(IN)        :: KDIM
      INTEGER                      :: NBTGRD
      INTEGER                      :: OUTBIT
      INTEGER                      :: OUTWRD
      INTEGER                      :: SCALE
      INTEGER,   INTENT(INOUT)     :: SECT1   ( 36 )
      INTEGER,   INTENT(IN)        :: SECT2   ( 25 )
      INTEGER,   INTENT(IN)        :: SIZE_3
      INTEGER,   INTENT(IN)        :: SECT3   ( SIZE_3 )
      INTEGER,   INTENT(IN)        :: S1OPT   ( 1 )
      INTEGER                      :: WORDNO

      REAL,      INTENT(IN)        :: BOTLEV  ( kdim )
      REAL,      INTENT(IN)        :: DUM3D   ( idim , jdim , kdim )
      REAL,      INTENT(IN)        :: TOPLEV  ( kdim )

      CHARACTER                    :: CSTAT
      CHARACTER*100		   :: MESSAGE(20)

!----------------------------------------------------------------------
!----------------------------------------------------------------------

      SCALE = 10 ** SECT1(24)

      DO K = 1, KDIM

        SECT1(8) = NINT(TOPLEV(K))
        SECT1(9) = NINT(BOTLEV(K))

        WORDNO   = 1
        BITNUM   = 0
        OUTWRD   = 0
        OUTBIT   = 0

         DO J=1, JDIM
           DO I=1, IDIM
             INGRID((J-1)*IDIM+I) = NINT(SCALE * DUM3D(I,J,K))
           ENDDO
         ENDDO

!----------------------------------------------------------------------
!--     CALL ROUTINE GRIBPR TO DETERMINE SIZE OF GRIB MESSAGE.
!--     THEN, ALLOCATE THE SIZE OF VARIABLE GRBOUT EXACTLY.
!----------------------------------------------------------------------

        CALL GRIBPR (WORDNO, BITNUM, INGRID, INFO, IMIN, NBTGRD, &
                     ISIZE, SECT1, ITOTBT, BUFFSZ)

        INFO(1)  = BUFFSZ

        ALLOCATE ( GRBOUT(BUFFSZ) )

!----------------------------------------------------------------------
!--     CALL GRIB ROUTINES TO BUILD GRIB MESSAGE. THEN WRITE IT
!--     THE GRIB FILE.
!----------------------------------------------------------------------

        !print*,WORDNO,BITNUM,OUTWRD,OUTBIT
        !print*,SECT1,S1OPT,SECT2,IMIN,NBTGRD,ISIZE,ITOTBT

        CALL GRIB (GRBOUT, WORDNO, BITNUM, OUTWRD, OUTBIT, INFO, &
                   SECT1, S1OPT, SECT2, SECT3, INGRID,           &
                   IMIN, NBTGRD, ISIZE, ITOTBT)

        WRITE (IGRIB, IOSTAT = ISTAT, ERR = 9000)  GRBOUT(1:OUTWRD)
   
        DEALLOCATE ( GRBOUT )

      ENDDO

      RETURN

!----------------------------------------------------------------------
!--   ERROR HANDLING SECTION.
!----------------------------------------------------------------------

 9000 PRINT*,' '
      PRINT*,'*********************************************'
      PRINT*,'*** SUBROUTINE: GRIBDAT'
      PRINT*,'***   ERROR WRITING GRIB FILE'
      PRINT*,'***   ISTAT IS ', ISTAT
      PRINT*,'*********************************************'
      WRITE(CSTAT,'(i9)',IOSTAT=ISTAT1) ISTAT
      STOP  

END SUBROUTINE GRIBDAT
